package dev.micle.totemofreviving.event;

import dev.micle.totemofreviving.TotemOfReviving;
import dev.micle.totemofreviving.config.Config;
import dev.micle.totemofreviving.item.StrawTotemItem;
import net.minecraft.client.Minecraft;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.item.crafting.RecipeManager;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.api.distmarker.OnlyIn;
import net.minecraftforge.event.entity.EntityJoinWorldEvent;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.config.ModConfig;

import java.util.ArrayList;

public class RecipeUpdateEventHandler {
	public static class ModBus {
		@SubscribeEvent
		@OnlyIn(Dist.CLIENT)
		public void onModConfigReloadEvent(ModConfig.Reloading event) {
			if (!event.getConfig().getModId().equals(TotemOfReviving.MOD_ID)) {
				return;
			}
			if (Minecraft.getInstance().getSingleplayerServer() != null) {
				updateRecipeManager(Minecraft.getInstance().getSingleplayerServer().getRecipeManager());
				return;
			}
			if (Minecraft.getInstance().getConnection() != null) {
				updateRecipeManager(Minecraft.getInstance().getConnection().getRecipeManager());
				return;
			}
			System.out.println("FAILED TO GET RECIPE MANAGER!");
		}
	}
	
	public static class EventBus {
		@SubscribeEvent
		@OnlyIn(Dist.CLIENT)
		public void onPlayerJoinEvent(EntityJoinWorldEvent event) {
			if (!(event.getEntity() instanceof PlayerEntity)) { return; }
			updateRecipeManager(event.getEntity().level.getRecipeManager());
		}
	}
	
	private static void updateRecipeManager(RecipeManager recipeManager) {
		ArrayList<IRecipe<?>> newRecipes = new ArrayList<>();
		for (IRecipe<?> recipe : recipeManager.getRecipes()) {
			if (recipe.getResultItem().getItem() instanceof StrawTotemItem) {
				StrawTotemItem.setRecipe(recipe);
				if (!Config.Server.getStrawTotemConfig().getIsEnabled()) {
					continue;
				}
			}
			newRecipes.add(recipe);
		}
		
		if (!newRecipes.contains(StrawTotemItem.getRecipe()) && Config.Server.getStrawTotemConfig().getIsEnabled()) {
			newRecipes.add(StrawTotemItem.getRecipe());
		}
		
		recipeManager.replaceRecipes(newRecipes);
	}
}
